#include "audiolayerwidget.h"


AudioLayerWidget::AudioLayerWidget(QWidget *parent, QString name):
    QGroupBox(parent)
  , m_suspendResumeButton(0)
  , m_volumeIndicator(new QSpinBox)
  , m_panIndicator(new QSpinBox)
  , m_pitchIndicator(new QSpinBox)
{
    this->setTitle(name);
    QVBoxLayout *layout = new QVBoxLayout;

    QGridLayout *status = new QGridLayout;
    m_statusLabel = new QLabel;
    m_statusLabel->setText(STATUS_LABEL);
    m_statusValue = new QLabel;
    status->addWidget(m_statusLabel, 0, 0);
    status->addWidget(m_statusValue, 0, 2);
    m_folderLabel = new QLabel;
    m_folderLabel->setText(FOLDER_LABEL);
    m_folderValue = new QLabel;
    m_folderValue->setMaximumWidth(200);
    status->addWidget(m_folderLabel, 1, 0);
    status->addWidget(m_folderValue, 1, 1);
    m_fileLabel = new QLabel;
    m_fileLabel->setText(FILE_LABEL);
    m_fileValue = new QLabel;
    m_fileValue->setMaximumWidth(200);
    status->addWidget(m_fileLabel, 1, 2);
    status->addWidget(m_fileValue, 1, 3);
    layout->addLayout(status);

    QGridLayout *volumeBox = new QGridLayout;
    m_volumeLabel = new QLabel;
    m_volumeLabel->setText(tr(VOLUME_LABEL));
    m_volumeSlider = new QSlider(Qt::Horizontal);
    m_volumeSlider->setMinimum(0);
    m_volumeSlider->setMaximum(100);
    m_volumeSlider->setSingleStep(1);
    m_volumeIndicator->setRange(0, 100);
    m_volumeIndicator->setValue(0);
    m_volumeIndicator->setMaximumWidth(40);
    m_volumeIndicator->setButtonSymbols(QAbstractSpinBox::NoButtons);
    volumeBox->addWidget(m_volumeLabel, 0, 0);
    volumeBox->addWidget(m_volumeSlider, 0, 1);
    volumeBox->addWidget(m_volumeIndicator, 0, 2);
    connect(m_volumeSlider, SIGNAL(valueChanged(int)), this, SLOT(volumeChanged(int)));
    connect(m_volumeSlider, &QSlider::valueChanged, this, [=] () {
        m_volumeIndicator->setValue(m_volumeSlider->value());
    });
    m_panLabel = new QLabel;
    m_panLabel->setText("Pan");
    m_panSlider = new QSlider(Qt::Horizontal);
    m_panSlider->setMinimum(0);
    m_panSlider->setMaximum(255);
    m_panSlider->setSingleStep(1);
    m_panIndicator->setRange(0, 255);
    m_panIndicator->setValue(128);
    m_panIndicator->setMaximumWidth(40);
    m_panIndicator->setButtonSymbols(QAbstractSpinBox::NoButtons);
    connect(m_panSlider, &QSlider::valueChanged, this, [=] () {
        m_panIndicator->setValue(m_panSlider->value());
    });
    connect(m_panSlider, SIGNAL(valueChanged(int)), this, SLOT(panChanged(int)));
    volumeBox->addWidget(m_panLabel, 1, 0);
    volumeBox->addWidget(m_panSlider, 1, 1);
    volumeBox->addWidget(m_panIndicator, 1, 2);
    m_pitchLabel = new QLabel;
    m_pitchLabel->setText("Pitch");
    m_pitchSlider = new QSlider(Qt::Horizontal);
    m_pitchSlider->setMinimum(0);
    m_pitchSlider->setMaximum(255);
    m_pitchSlider->setSingleStep(1);
    m_pitchIndicator->setRange(0, 255);
    m_pitchIndicator->setValue(128);
    m_pitchIndicator->setMaximumWidth(40);
    m_pitchIndicator->setButtonSymbols(QAbstractSpinBox::NoButtons);
    connect(m_pitchSlider, &QSlider::valueChanged, this, [=] () {
        m_pitchIndicator->setValue(m_pitchSlider->value());
    });
    connect(m_pitchSlider, SIGNAL(valueChanged(int)), this, SLOT(pitchChanged(int)));
    volumeBox->addWidget(m_pitchLabel, 2, 0);
    volumeBox->addWidget(m_pitchSlider, 2, 1);
    volumeBox->addWidget(m_pitchIndicator, 2, 2);
    layout->addLayout(volumeBox);

    QHBoxLayout *progressTime = new QHBoxLayout;
    m_progressTimeLabel = new QLabel;
    m_progressTimeLabel->setText(PROGRESS_TIME_LABEL);
    m_progressTime = new QTimeEdit;
    m_progressTime->text();
    m_progressTime->setDisplayFormat("h:mm:ss:zzz");
    m_progressTime->setReadOnly(true);
    m_progressTime->setButtonSymbols(QAbstractSpinBox::NoButtons);
    m_progressTime->setMaximumWidth(120);
    progressTime->addWidget(m_progressTimeLabel);
    progressTime->addWidget(m_progressTime);
    m_totalTimeLabel = new QLabel;
    m_totalTimeLabel->setText(TOTAL_TIME_LABEL);
    m_totalTimeValue = new QTimeEdit;
    m_totalTimeValue->setDisplayFormat("h:mm:ss:zzz");
    m_totalTimeValue->setReadOnly(true);
    m_totalTimeValue->setButtonSymbols(QAbstractSpinBox::NoButtons);
    m_totalTimeValue->setMaximumWidth(120);
    progressTime->addWidget(m_totalTimeLabel);
    progressTime->addWidget(m_totalTimeValue);
    layout->addLayout(progressTime);

    m_progressSlider = new QSlider(Qt::Horizontal);
    layout->addWidget(m_progressSlider);
    m_suspendResumeButton = new QPushButton(this);
    m_suspendResumeButton->setText(tr(SUSPEND_LABEL));
    connect(m_suspendResumeButton, SIGNAL(clicked()), SLOT(toggleSuspendResume()));
    layout->addWidget(m_suspendResumeButton);

    this->setLayout(layout);
}

AudioLayerWidget::~AudioLayerWidget()
{

}

// From UI.
void AudioLayerWidget::volumeChanged(int value)
{
    (void)value;
    // ToDo: call the audio engine
}

void AudioLayerWidget::panChanged(int value)
{
    (void)value;
    // ToDo: call the audio engine
}

void AudioLayerWidget::pitchChanged(int value)
{
    (void)value;
    // ToDo: call the audio engine
}

void AudioLayerWidget::loopChanged(int value)
{
    (void)value;
    // ToDo: call the audio engine
}

void AudioLayerWidget::toggleSuspendResume()
{
    switch (m_status) {
        case Status::PlayingLoop:
        case Status::PlayingOnce:
            this->setPlaybackStatus(Status::Paused);
        case Status::Paused:
        case Status::Stopped:
            this->setPlaybackStatus(Status::PlayingOnce);
    }
    // ToDo: call the audio engine
}

// from DMX signals
void AudioLayerWidget::setVol(qreal vol)
{
    m_volumeSlider->blockSignals(true);
    m_volumeSlider->setValue(vol);
    m_volumeIndicator->setValue(vol);
    m_volumeSlider->blockSignals(false);
}

void AudioLayerWidget::setPan(qreal pan)
{
    m_panSlider->blockSignals(true);
    m_panSlider->setValue(pan);
    m_panIndicator->setValue(pan);
    m_panSlider->blockSignals(false);
}

void AudioLayerWidget::setPitch(qreal pitch)
{
    m_pitchSlider->blockSignals(true);
    m_pitchSlider->setValue(pitch);
    m_pitchIndicator->setValue(pitch);
    m_pitchSlider->blockSignals(false);
}

void AudioLayerWidget::fileLoaded(QString file)
{
    QStringList list = file.split("/");
    int size = list.size();
    if (size >= 2) {
       m_folderValue->setText(list.at(size - 2));
       m_fileValue->setText(list.at(size - 1));
    }
}

void AudioLayerWidget::setPlaybackStatus(Status status)
{
    m_status = status;
    if (status == Status::Stopped)
        m_progressTime->setTime(QTime::fromMSecsSinceStartOfDay(0));
    QString tmp;
    switch (status) {
    case Status::Paused:
        tmp.append("Paused");
        break;
    case Status::PlayingLoop:
        tmp.append("Playing Loop");
        break;
    case Status::PlayingOnce:
        tmp.append("Playing one");
        break;
    case Status::Stopped:
        tmp.append("Stopped");
        break;
    }
    m_statusValue->setText(tmp);
    m_suspendResumeButton->setText(tmp);
}

void AudioLayerWidget::durationChanged(qint64 dur)
{
    dur *= 1000;
    m_progressSlider->setMaximum(dur);
    m_totalTimeValue->setTime(QTime::fromMSecsSinceStartOfDay(dur));
}

void AudioLayerWidget::refreshUi(float progress)
{
     progress *= 1000;
     m_progressSlider->setValue(progress);
     m_progressTime->setTime(QTime::fromMSecsSinceStartOfDay(progress));
}
