#ifndef AUDIOLAYERWIDGET_H
#define AUDIOLAYERWIDGET_H

#include <iostream>

#include <QDebug>
#include <QTime>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QPushButton>
#include <QSlider>
#include <QTimer>
#include <QTimeEdit>
#include <QGroupBox>
#include <QCheckBox>
#include <QSpinBox>

#include "defines.h"

class AudioLayerWidget : public QGroupBox
{
    Q_OBJECT
    
public:
    explicit AudioLayerWidget(QWidget *parent = 0, QString name = "Layer");
    ~AudioLayerWidget();
    void setVol(qreal vol);
    void resume();
    void setPan(qreal pan);
    void setPitch(qreal pitch);
    void setLoop(bool on);
    void setPlaybackStatus(Status status);
    inline Status getPlaybackStatus() { return m_status; }

private:
    QPushButton *m_suspendResumeButton;

    QLabel *m_statusLabel;
    QLabel * m_statusValue;
    QLabel *m_fileLabel;
    QLabel *m_fileValue;
    QLabel * m_folderLabel;
    QLabel * m_folderValue;

    QLabel *m_volumeLabel;
    QSlider *m_volumeSlider;
    QSpinBox *m_volumeIndicator;
    QLabel *m_panLabel;
    QSlider *m_panSlider;
    QSpinBox *m_panIndicator;
    QLabel *m_pitchLabel;
    QSlider *m_pitchSlider;
    QSpinBox *m_pitchIndicator;

    QLabel * m_progressLabel;
    QSlider *m_progressSlider;
    QLabel *m_progressTimeLabel;
    QTimeEdit *m_progressTime;
    QLabel *m_totalTimeLabel;
    QTimeEdit *m_totalTimeValue;

    Status m_status;

public slots:
    void toggleSuspendResume();
    void volumeChanged(int vol);
    void panChanged(int vol);
    void pitchChanged(int vol);
    void loopChanged(int vol);
    void fileLoaded(QString file);
    void durationChanged(qint64 dur);
    void refreshUi(float progress);
};

#endif // AUDIOLAYERWIDGET_H
