#include "medialibrary.h"

MediaLibrary *MediaLibrary::_instance = 0;

MediaLibrary *MediaLibrary::getInstance() {

    if (_instance == 0) {
        _instance = new MediaLibrary();
        Q_CHECK_PTR(_instance);
    }
    return _instance;
}

MediaLibrary::MediaLibrary(QObject *parent) :
    QObject(parent)
{
    m_media = NULL;
}

void MediaLibrary::initMediaLibrary() {
    QDir dir;
    if (!dir.cd(Settings::getInstance()->getPathMedia())) {
        qWarning("Can not cd to the path: %s", Settings::getInstance()->getPathMedia().toLatin1().constData());
        return;
    }
    m_media = new QList<MediaFolder>;
    dir.setFilter(QDir::Dirs | QDir::NoSymLinks | QDir::NoDotAndDotDot);
    QFileInfoList filelist = dir.entryInfoList();
    dir.setFilter(QDir::Files);
    QFileInfo fileInfo;
    struct MediaFolder mediai;
    for (int i = 0; i < filelist.size(); ++i) {
        fileInfo = filelist.at(i);
        QString name = fileInfo.absoluteFilePath();
        dir.cd(fileInfo.baseName());
        mediai.m_Id = i;
        mediai.m_Name = name;
        mediai.m_ElementCount = dir.count();
        mediai.m_MediaInformation = getMediaInformation(dir);
        dir.cdUp();
        m_media->append(mediai);
    }
}

/**
 * fill the struct with all files in one library/folder
 */
QList<MediaFile> MediaLibrary::getMediaInformation(QDir dir)
{
    QList<MediaFile> mediaList;
    MediaFile mediainf;
    dir.setFilter(QDir::Files);
    QFileInfoList filelist = dir.entryInfoList();
    QFileInfo fileInfo;
    for (int i = 0; i < filelist.size(); ++i) {
        fileInfo = filelist.at(i);
        // Update the data base with the new file
        mediainf.Number = i;
        mediainf.MediaName = fileInfo.absoluteFilePath();
        mediainf.MediaLength = 1000;
        mediaList.append(mediainf);
    }
    return mediaList;
}

/**
 * returns the path to a media file from the library.
 */
QString MediaLibrary::requestNewFile(int folder, int file){
    if (!m_media) {
        qWarning("MediaLibrary is not init. Set a correct path to media library");
        return NULL;
    }
    QString newfile;
    if (folder < m_media->size()) {
        if (file < m_media->at(folder).m_MediaInformation.size()) {
           newfile = m_media->at(folder).m_MediaInformation.at(file).MediaName;
        } else
           qInfo("requestNewFile: Requested file %i is greater than files in library %i", file, m_media->at(folder).m_MediaInformation.size());
    } else
        qInfo("requestNewFile: Requested folder %i is greater than media libraries %i", folder, m_media->size());
    return newfile;
}
