#ifndef MEDIALIBRARY_H
#define MEDIALIBRARY_H

#include <QObject>
#include <QDir>
#include <QDebug>

#include "defines.h"
#include "settings.h"

class MediaLibrary : public QObject
{

    Q_OBJECT

    friend class libreMediaserverAudio; // Not working... WHY?
    friend class Settings; // working...

public:

    static MediaLibrary *getInstance();

    /**
     * @brief request a new file from the media library
     * @param int folder - the folder required
     * @param int layer - file required
     * @return QString the file required with full path
     */
    QString requestNewFile(int folder, int layer);

    /**
     * @brief Inicia la biblioteca de medios
     */
    void initMediaLibrary();

protected:

    /**
    * @brief Reload the media library on change of path or change the media files
    */
    inline void rescanMediaLibrary() {
       if (m_media != NULL)
           delete m_media;
       initMediaLibrary();
   }

private:

    explicit MediaLibrary(QObject *parent = 0);
    static MediaLibrary *_instance;

// Wich structure is more efficient for us?
// QList, QMap, QSet, QVector?
    QList<MediaFolder> *m_media; // Library to save the folders/media libraries and index each media file inside

    /**
     * @brief Returns a list withe the medias contained in a folder
     * @param QDir - the directory when are the media
     * @return QList<MediaFile> The list with the medias in a folder
     */
    QList<MediaFile> getMediaInformation(QDir dir); // Get all the information of each media file in a dir

signals:
    
public slots:
    
};

#endif // MEDIALIBRARY_H
