#ifndef OLATHREAD_H
#define OLATHREAD_H

#include <string>

#include <QObject>
#include <QThread>
#include <QDebug>

#include <ola/DmxBuffer.h>
#include <ola/Logging.h>
#include <ola/OlaClientWrapper.h>
#include <ola/client/OlaClient.h>
#include <ola/DmxBuffer.h>
#include <ola/client/ClientTypes.h>
#include <ola/Callback.h>

#include "defines.h"
#include "dmxPersonality.h"
#include "settings.h"

class olaThread : public QThread
{
    Q_OBJECT

public:

    olaThread(QObject *parent = 0);
    virtual ~olaThread();

    /** Retorna el valor de un canal
     *@param int layer the layer for we want the channel
     *@param int channel the channel for the value wanted
     *@return int the value
     */
    inline int getValue(int layer, int channel) {
        return m_dmx[layer][channel];
    }

    /**
     * @brief resendDMX emite todo el buffer DMX
     */
    void resendDmx();

private:

    void run ();
    ola::client::OlaClientWrapper *m_clientWrapper;
    ola::client::OlaClient *m_client;
    unsigned int m_counter;
    struct timeval m_last_data; // Last DMX frame received

    int m_dmx[LAYERS_NUMBER][LAYER_CHANNELS]; // DMX Buffer. Habría que cambiarlo si queremos hacer las capas dinámicas

    /**
     *  @brief Callback from ola. Control de errores en el registro de Universos en OLA
     *         typedef SingleUseCallback1<void, const Result&> ola::client::SetCallback
     *  @param ola::client::Result &error
     *  @return void
     *
     *
     */
    inline void RegisterComplete(const ola::client::Result &error) {
        if (error.Success()) {
            qDebug("Register Universe success");
            emit toTerminal("Register Universe success");
        } else {
            qWarning("Register command failed: %s", error.Error().c_str());
            emit toTerminal("olaThread| Register command failed " +  QString::fromStdString(error.Error()));
        }
      }

    /**
     * @brief Check if the dmx info is arriving each 4 seconds
     * @return bool
     */
    bool CheckDataLoss();

    /**
     * @brief RepeteableDMXCallBack from ola called when arrives a new dmx frame
     * typedef Callback2<void, const DMXMetadata&, const DmxBuffer&> ola::client::RepeatableDMXCallback
     * This is called one for second if there is not updated in the DMX frame. We need emit only the channels that
     * has changed to save resources.
     *
     *
     */
    void NewDmx(const ola::client::DMXMetadata &dmx_meta, const ola::DmxBuffer &buffer); //

    /**
     * @brief Sometimes the ola server closes the connection. This is a callback to handle this event an reconect to ola
     *
     *
     */
    void socketClosed();

    /**
     * @brief Open the connection with olad and start processing data.
     *
     *
     */
     void init();

public slots:

    void stop();  // Close the connection with olad.

    /**
     * @brief register one Universe
     * void ola::client::OlaClient::RegisterUniverse(unsigned int universe,RegisterAction register_action,SetCallback * callback
     * @param universe
     */
    inline void registerUniverse(int universe) {
       qDebug("Registering universe %d", universe);
       m_client->RegisterUniverse(universe,
                                  ola::client::REGISTER,
                                  ola::NewSingleCallback
                                  (this, &olaThread::RegisterComplete));
    }

    /**
     * @brief Register all the universes again
     *
     */
    inline void registerUniverse() {
        QSet<int> unis = Settings::getInstance()->getUniverses();
        foreach (const int &universe, unis) {
            registerUniverse(universe);
        }
    }

protected slots:

signals:

//    void finished(); // Signal for closing. Not used now.
    void dmxOutput(int layer, int channel, int value); // Signal when a channel has changed
    void toTerminal(QString message);
    void universeReceived(uint universe);
    void layerReceived();
};

using namespace ola;

#endif // OLATHREAD_H
